#include <windows.h>
#include <MotionPoint.h>
#include <Bone.h>

#include <math.h>
#include <crtdbg.h>


CMotionPoint::CMotionPoint()
{
	InitParams();
}
CMotionPoint::~CMotionPoint()
{
	DestroyObjs();
}

int CMotionPoint::InitParams()
{
	m_frame = 0.0;
	m_eul = D3DXVECTOR3( 0.0f, 0.0f, 0.0f );
	m_q.SetParams( 1.0f, 0.0f, 0.0f, 0.0f );
	m_totalq.SetParams( 1.0f, 0.0f, 0.0f, 0.0f );
	m_tra = D3DXVECTOR3( 0.0f, 0.0f, 0.0f );
	D3DXMatrixIdentity( &m_mat );
	D3DXMatrixIdentity( &m_totalmat );
	D3DXMatrixIdentity( &m_worldmat );

	m_prev = 0;
	m_next = 0;

	return 0;
}
int CMotionPoint::DestroyObjs()
{

	return 0;
}

int CMotionPoint::GetEul( D3DXVECTOR3* dsteul )
{
	*dsteul = m_eul;

	return 0;
}
int CMotionPoint::SetEul( D3DXVECTOR3 srceul )
{
	m_eul = srceul;
	m_q.SetRotation( srceul );

	return 0;
}

int CMotionPoint::MakeMat( CBone* srcbone )
{
	D3DXMATRIX befrotmat, aftrotmat, rotmat, tramat;
	D3DXMatrixIdentity( &befrotmat );
	D3DXMatrixIdentity( &aftrotmat );
	D3DXMatrixIdentity( &rotmat );
	D3DXMatrixIdentity( &tramat );

	befrotmat._41 = -srcbone->m_vertpos[ BT_PARENT ].x;
	befrotmat._42 = -srcbone->m_vertpos[ BT_PARENT ].y;
	befrotmat._43 = -srcbone->m_vertpos[ BT_PARENT ].z;

	aftrotmat._41 = srcbone->m_vertpos[ BT_PARENT ].x;
	aftrotmat._42 = srcbone->m_vertpos[ BT_PARENT ].y;
	aftrotmat._43 = srcbone->m_vertpos[ BT_PARENT ].z;

	rotmat = m_q.MakeRotMatX();

	tramat._41 = m_tra.x;
	tramat._42 = m_tra.y;
	tramat._43 = m_tra.z;

	m_mat = befrotmat * rotmat * aftrotmat * tramat;

	return 0;
}
int CMotionPoint::MakeTotalMat( D3DXMATRIX* parmat, CBone* srcbone )
{

	MakeMat( srcbone );

	m_totalmat = m_mat * *parmat;

	return 0;
}

int CMotionPoint::MakeWorldMat( D3DXMATRIX* wmat )
{
	m_worldmat = m_totalmat * *wmat;

	return 0;
}

int CMotionPoint::UpdateMatrix( D3DXMATRIX* wmat, D3DXMATRIX* parmat, CBone* srcbone )
{
	MakeTotalMat( parmat, srcbone );
	MakeWorldMat( wmat );

	return 0;
}

int CMotionPoint::AddToPrev( CMotionPoint* addmp )
{
	CMotionPoint *saveprev, *savenext;
	saveprev = m_prev;
	savenext = m_next;

	addmp->m_prev = m_prev;
	addmp->m_next = this;

	m_prev = addmp;

	if( saveprev ){
		saveprev->m_next = addmp;
	}

	return 0;
}

int CMotionPoint::AddToNext( CMotionPoint* addmp )
{
	CMotionPoint *saveprev, *savenext;
	saveprev = m_prev;
	savenext = m_next;

	addmp->m_prev = this;
	addmp->m_next = savenext;

	m_next = addmp;

	if( savenext ){
		savenext->m_prev = addmp;
	}

	return 0;
}

int CMotionPoint::LeaveFromChain( int srcmotid, CBone* boneptr )
{
	CMotionPoint *saveprev, *savenext;
	saveprev = m_prev;
	savenext = m_next;

	m_prev = 0;
	m_next = 0;

	if( saveprev ){
		saveprev->m_next = savenext;
	}

	if( savenext ){
		savenext->m_prev = saveprev;
	}

	if( (srcmotid >= 0) && boneptr && !saveprev ){
		boneptr->m_motionkey[srcmotid] = savenext;
	}

	return 0;
}

