#ifndef MODELH
#define MODELH

#include <d3dx9.h>
#include <wchar.h>
#include <Coef.h>
#include <string>
#include <map>

#include <OrgWindow.h>

using namespace std;


class CMQOMaterial;
class CMQOObject;
class CMQOFace;
class CBone;
class CInfScope;
class CMySprite;
class CMotionPoint;

class CModel
{
public:
	CModel();
	~CModel();

	int LoadMQO( LPDIRECT3DDEVICE9 pdev, WCHAR* wfile, int ismedia );
	int OnRender( LPDIRECT3DDEVICE9 pdev, int lightflag, D3DXVECTOR4 diffusemult, CModel* bmarkptr, CMySprite* bcircleptr );
	int GetModelBound( MODELBOUND* dstb );

	int MakeObjectName();
	int MakeBoneTri();
	int MakeInfScope();
	int MakePolyMesh3();
	int MakeExtLine();
	int MakeDispObj();

	int CalcInf();

	int DbgDump();

	int UpdateMatrix( D3DXMATRIX* wmat, D3DXMATRIX* vpmat );
	int SetShaderConst();

	int SetBoneEul( int boneno, D3DXVECTOR3 srceul );
	int GetBoneEul( int boneno, D3DXVECTOR3* dsteul );

	int FillTimeLine( OrgWinGUI::OWP_Timeline& timeline, map<int, int>& lineno2boneno, map<int, int>& boneno2lineno );

	int AddMotion( char* srcname, double srcleng, int* dstid );
	CMotionPoint* AddMotionPoint( int srcmotid, double srcframe, int srcboneno, int calcflag, int* existptr );

	int SetCurrentMotion( int srcmotid );
	int SetMotionFrame( double srcframe );
	int GetMotionFrame( double* dstframe );
	int SetMotionSpeed( double srcspeed );
	int GetMotionSpeed( double* dstspeed );

	int DeleteMotion( int motid );
	int GetSymBoneNo( int srcboneno, int* dstboneno, int* existptr );

private:
	int InitParams();
	int DestroyObjs();
	int CreateMaterialTexture();

	int AddModelBound( MODELBOUND* mb, MODELBOUND* addmb );

	int DestroyMaterial();
	int DestroyObject();
	int DestroyAncObj();
	int DestroyAllMotionInfo();

	void MakeBoneReq( CBone* parbone, CMQOFace* curface, D3DXVECTOR3* pointptr, int broflag, int* errcntptr );

	int SetShapeNoReq( CMQOFace** ppface, int facenum, int searchp, int shapeno, int* setfacenum);
	int SetFaceOfShape( CMQOFace** ppface, int facenum, int shapeno, CMQOFace** ppface2, int setfacenum );

	int DbgDumpBoneReq( CBone* boneptr, int broflag );

	void UpdateMatrixReq( int srcmotid, double srcframe, D3DXMATRIX* wmat, D3DXMATRIX* parmat, CBone* srcbone, int broflag );
	int RenderBoneMark( LPDIRECT3DDEVICE9 pdev, CModel* bmarkptr, CMySprite* bcircleptr );

	void FillTimelineReq( OrgWinGUI::OWP_Timeline& timeline, CBone* curbone, int* linenoptr, 
		map<int, int>& lineno2boneno, map<int, int>& boneno2lineno, int broflag );


public:
	int m_modelno;
	LPDIRECT3DDEVICE9 m_pdev;

	map<int, CMQOMaterial*> m_material;
	map<int, CMQOObject*> m_object;
	map<string, CMQOObject*> m_objectname;

	map<int, CMQOMaterial*> m_ancmaterial;
	map<int, CMQOObject*> m_ancobject;
	map<int, CMQOObject*> m_boneobject;
	map<string, CBone*> m_bonename;

	map<int, CBone*> m_bonelist;

	int m_topbonenum;
	CBone* m_topbone;

	map<int, CInfScope*> m_infscope;


	WCHAR m_filename[MAX_PATH];
	WCHAR m_dirname[MAX_PATH];

	D3DXMATRIX m_bonemat[ MAXBONENUM ];

	D3DXMATRIX m_matWorld;
	D3DXMATRIX m_matVP;

	map<int, MOTINFO*> m_motinfo;
	MOTINFO* m_curmotinfo;
};

#endif